/*
 * Copyright (C) 2023 Associated Universities, Inc. Washington DC, USA.
 *
 * This file is part of NRAO Workspaces.
 *
 * Workspaces is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Workspaces is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
 */

package db

import (
    "database/sql"
    "fmt"
    "log"
    "strings"
    _ "github.com/lib/pq"
    "gitlab.nrao.edu/ssa/gocapo/capo/config"
)

// Contains all necessary data to connect to the database
type DbInfo struct {
    Host        string
    Port        int
    User        string
    Password    string
    Dbname      string
    Profile     string
    Prefix      string
    IsSsl       bool
    CapoPath string
}


// Check if an error happened and panic on it if so
//
// @param err An error object to report
func CheckError(err error) {
    if err != nil {
        log.Fatalln(err)
    }
}


// Pull the login information for the database from a CAPO profile
//
// @param db_info a DbInfo type with information to connect to the database
// @return a DbInfo type holding db_info's data with the User and Password
// fields populated
func getDbLoginFromProperties(connectionInfo DbInfo) DbInfo {
    prop, err := config.InitConfig(connectionInfo.Profile, connectionInfo.CapoPath)
    CheckError(err)

    settings := prop.SettingsForPrefix(connectionInfo.Prefix)

    connectionInfo.User = settings.GetString("jdbcUsername")
    connectionInfo.Password = settings.GetString("jdbcPassword")
    url := settings.GetString("jdbcUrl")
    url = strings.TrimPrefix(url, "jdbc:postgresql://")
    splitUrl := strings.Split(url, "/")
    connectionInfo.Host = splitUrl[0]

    // Remove extraneous port on hostname if it exists
    connectionInfo.Host = strings.Split(connectionInfo.Host, ":")[0]

    connectionInfo.Dbname = splitUrl[1]
    connectionInfo.Port = 5432
    connectionInfo.IsSsl = false
    return connectionInfo
}

// Establish a connection to the database
//
// @param db_info A DbInfo type with information to connect to the database
// @return A DB object with an open connection to the database
func GetConnection(db_info DbInfo) *sql.DB {
    // Get db info and build string to get connection
    db_info = getDbLoginFromProperties(db_info)

    ssl_mode := "disable"
    if db_info.IsSsl {
        ssl_mode = "require"
    }
    conn_info := fmt.Sprintf("host=%s port=%d user=%s password=%s dbname=%s sslmode=%s", db_info.Host, db_info.Port, db_info.User, db_info.Password, db_info.Dbname, ssl_mode)

    db, err := sql.Open("postgres", conn_info)
    CheckError(err)

    err = db.Ping()
    CheckError(err)

    return db
}
