#
# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
"""
The validation system is fairly small and straightforward. Each FileValidator
is given whatever special parameters it needs in the constructor, and then
exposes a validate method that validates a file path according to whatever
its special sauce is.

We have three of these at the moment:

 - SizeValidator ensures that the file is the expected size (in bytes)
 - Crc32Validator calculates the crc32 of a file and makes sure it matches
   the expected value. NGAS keeps these so we happen to have them on-hand
 - XmlValidator ensures that an XML file is well-formed

"""

# pylint: disable=E0401, E0402, R0201, R0903
import binascii
import xml.sax
from pathlib import Path

from .exceptions import FileValidationFault
from .interfaces import FileValidator


class SizeValidator(FileValidator):
    """
    Validates the size of a file in bytes. If the expected and actual sizes match,
    the file validates.
    """

    def __init__(self, size: int):
        self.size = size

    def validate(self, path: Path):
        # check the size
        actual_size = path.stat().st_size
        if actual_size != self.size:
            raise FileValidationFault(f"File size mismatch: expected {self.size} but received {actual_size}", path)


class Crc32Validator(FileValidator):
    """
    Validates the cksum of a file. This is an algorithm used by NGAS.
    If the content of the file generates the same cksum we expect,
    the file passes validation.
    """

    def __init__(self, checksum: int):
        self.checksum = checksum

    def validate(self, path: Path):
        # there are a couple ways to do this, another is with the zlib module
        # the important thing is reading this a chunk at a time
        checksum = binascii.crc32(b"")
        with path.open("rb") as content:
            # read1() guarantees that only a single system read() call is made
            # this should be more efficient than reading the entire thing into memory at once
            # 0 is returned when we run out of stuff to read
            while len(chunk := content.read1()) != 0:
                checksum = binascii.crc32(chunk, checksum)

        # the magic bitmath here is courtesy of the Python documentation:
        # Python 2 generated signed numbers here, but Python 3 generates unsigned numbers instead
        if checksum != (self.checksum & 0xFFFFFFFF):
            raise FileValidationFault(
                f"CRC32 failed: expected {self.checksum & 0xFFFFFFFF}, got {checksum} instead", path
            )


class XmlWellFormedValidator(FileValidator):
    """
    Validates that a file is well-formed XML. ALMA SDM files happen
    to be XML and can be validated this way.
    """

    def validate(self, path: Path):
        try:
            parser = xml.sax.make_parser()
            parser.setContentHandler(xml.sax.handler.ContentHandler())
            parser.parse(path)
        except Exception as exc:
            raise FileValidationFault("XML is not well-formed", path) from exc
